
INSTRUCTION_TUNING_PROMPT_COMPONENTS = {
    "task_description": (
        "Below is an instruction that describes a task, with or without input. "
        "Write a response that appropriately completes the request.\n\n"
    ),
    "instruction": "### Instruction:\n{}\n\n",
    "input": "### Input:\n{}\n\n",
    "response": "### Response:\n{}\n\n",
    "no_response": "### Response:\n\n",
}